
#include <sys/signal.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <time.h>
#include <fcntl.h>
#include <limits.h>
#include <errno.h>
#include <termios.h>
#include <asm/byteorder.h>
#include <curses.h>

/* Colors */
#define NC_LIGHTGRAY          COLOR_PAIR( 20)
#define NC_WHITE            ( COLOR_PAIR( 21) | A_BOLD)
#define NC_BG_LIGHTGRAY       COLOR_PAIR( 22)
#define NC_COLOR_RED          COLOR_PAIR( 23)
#define NC_COLOR_ATTENTION  ( COLOR_PAIR( 23) | A_BOLD)
#define NC_COLOR_BACKCYAN     COLOR_PAIR( 24)
#define NC_CYANBLUE           COLOR_PAIR( 25)
#define NC_COLOR_BACKBLUE   ( COLOR_PAIR( 25) | A_BOLD)
#define NC_COLOR_BACKGRAY     COLOR_PAIR( 26)

#define BUF_BBLUE  0
#define BUF_BLGRAY 1
#define BUF_WHITE  2

/* Normal state */
#define TERM_COLOR_OFF      A_NORMAL

#define FLAG_EDIT           2
#define FLAG_BELL           3
#define FLAG_NOHEX          5
#define FLAG_READONLY       6

#define EDIT_BITOP_XOR  0
#define EDIT_BITOP_OR   1
#define EDIT_BITOP_AND  2
#define EDIT_BITOP_NEG  3
#define EDIT_BITOP_ADD  4
#define EDIT_BITOP_SUB  5

#define EDIT_ASK_SAVE   0
#define EDIT_ASK_QUIT   1

/*
 * Global variables
 */

/* note: extern variables are explained where defined, just use grep. */

extern int fd;
extern struct stat stbuf;

extern int flag_bell;
extern int flag_edit;
extern int flag_nohex;
extern int flag_readonly;

extern int bytes_to_read;
extern int bytes_per_row;
extern int bytes_per_row_hex;

extern int ex, ey;

extern int bread;

/* The double linked list file structure */
struct ll_file {
        char *filename;
        long filepos;
        struct ll_file *next;
        struct ll_file *prev;
};

extern struct ll_file *ll_file_head;
extern struct ll_file *ll_file_current;
extern struct ll_file *ll_file_tail;
extern int ll_file_qty;

extern struct ll_file *ll_file_browse_curr;
extern struct ll_file *ll_file_browse_top;
extern int ll_file_browse_pos;

/* The linked list for editing changes */
struct ll_edit {
       char byte;
       long filepos;
       struct ll_edit *next;
};

extern struct ll_edit *ll_edit_head;

extern char hex_trans[];
extern char *obuffer;

extern long fpos;

extern int edit_mask;

extern char srch[];

extern char *versionstr;

/* edit.c */
void edit( void);
void set_char_colorxy( char byte, int what_attr);

/* file.c */
#define file_getpos() lseek( fd, (long) 0, SEEK_CUR);
int file_go_next( void);
int file_go_prev( void);
void file_open( char *filename);
void file_cannot_open( void);
void file_next_line( void);
void file_prev_line( void);
void file_next_byte( void);
void file_prev_byte( void);
void file_go_home( void);
void file_go_end( void);
void file_next_block( void);
void file_prev_block( void);

/* help.c */
void help( void);
void show_gnu_license( void);
void ascii_table( void);

/* hex.c */
void hex_mainloop( int view_or_edit);
void hex_showfile( void);
void hex_fill_hex_buffer( char *buf, char *hex_buffer, int size);
void hex_fill_offset_buffer( char *offset_buffer, int pos);
void hex_fill_ascii_buffer( char *buf, char *ascii_buffer, int size);

/* init.c */
void init( void);
void init_ncurses( void);
void init_buffers( void);

/* keyboard.c */
void key_scanf( WINDOW *w, char *str, int leng);

/* linkedlist.c */
void ll_file_init( void);
struct ll_file *ll_file_alloc( char *filename);
void ll_file_add( char *filename);
void ll_file_insert( char *filename);
void ll_file_free( void);
struct ll_file *ll_file_find( char *filename);
void ll_file_delete( void);
void ll_file_browse( void);
void ll_file_browse_show( WINDOW *w);
int ll_file_browse_next( int nitems);
int ll_file_browse_back( int nitems);
struct ll_edit *ll_edit_alloc( char byte);
void ll_edit_add( char byte, long filepos, struct ll_edit *tmp);
void ll_edit_free( void);

/* misc.c */
void do_exit( int status);
void draw_filename( char *filename);
void draw_main_screen( void);
void draw_fkeys( void);
void draw_offset_n_perc( long offset, long filesize);
void set_flag( int flag);
void *allocate_mem( int size);
char *trim_spaces( char *s);
void set_hexmode_variables( void);
int check_readonly( void);
char *replicate( char c, int size);
void translate_buffer_colors( int what_attr, attr_t *attr, short *color);

/* nclib.c */
WINDOW *nc_box( int x, int y, int xlen, int ylen, chtype attr);
void nc_color_wputs( WINDOW *w, int x, int y, int size, char *str, chtype attr);
void nc_color_puts( int x, int y, int size, char *str, chtype attr);
void nc_color_buffer_puts( int x, int y, int size, char *str, int what_attr);
void nc_color_wputc( WINDOW *w, int x, int y, chtype byte, chtype attr);
void nc_color_putc( int x, int y, chtype byte, chtype attr);
WINDOW *nc_dialog_box( char *titlestr, char *valuestr, int size, chtype tattr, chtype vattr);
void nc_fill_screen( chtype attr);
void nc_show_alert_box( char *str, int addy);
void nc_show_screentext( char *text, char *title);
void nc_do_title( WINDOW *w, char *title);
void nc_put_bottom_line( WINDOW *w, char *text);
void nc_flush( void);
int nc_ask( char *msg, char *q1, char *q2, char *q3, int posit);
WINDOW *nc_show_message( char *msg, int addy);
void nc_fill_obuffer( void);

/* term.c */
void term_init( void);
void term_beep( void);
void term_unhide_cursor( void);
void term_hide_cursor( void);
void term_goto( int x, int y);
void term_wgoto( WINDOW *w, int x, int y);

/* search.c */
void search( void);
void search_again( void);
void filegoto( void);
void search_set_color( void);

/* signals.c */
void signals_init( void);

