/* 
   File routines
   --------------------------------------------------------------------
   VCHE NC - Virtual Console Hex Editor NCURSES version

   Copyright (C) 1998 Diego Javier Grigna <diego@grigna.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"

int file_go_next( void)
{
 struct ll_file *p;

 if( ll_file_current == NULL) {
     nc_show_alert_box( "No more files", 0);
     do_exit( 1);
 }

 if( ll_file_current->next == NULL) {
     nc_show_alert_box( "No next file", 0);
     return -1;
 }

 p = ll_file_current;
 ll_file_current = p->next;

 file_open( ll_file_current->filename);

 return 1;
}

int file_go_prev( void)
{
 struct ll_file *p;

 if( ll_file_current == NULL) {
     nc_show_alert_box( "No more files", 0);
     do_exit( 1);
 }

 if( ll_file_current->prev == NULL) {
     nc_show_alert_box( "No previous file", 0);
     return -1;
 }

 p = ll_file_current;
 ll_file_current = p->prev;

 file_open( ll_file_current->filename);

 return 1;
}

void file_open( char *filename)
{
 char buf[ 2048];
 int open_flags;

 if( fd != fileno(  stdin) &&
     fd != fileno( stderr) &&
     fd != fileno( stdout) )
     close( fd);

reopenit:

 if( flag_readonly)
     open_flags = O_RDONLY;
 else
     open_flags = O_RDWR;

 if( (fd = open( filename, open_flags)) == -1) {
     if( !flag_readonly) {
         flag_readonly = 1;
         set_flag( FLAG_READONLY);
         goto reopenit;
     }
     sprintf( buf, "Can't open \"%s\" : %s", filename, strerror( errno));
     nc_show_alert_box( buf, 0);
     file_cannot_open();
     return;
 }

 fstat( fd, &stbuf);

 if( ( stbuf.st_mode & S_IFMT) == S_IFDIR){
     sprintf( buf, "\"%s\" is a directory", filename);
     nc_show_alert_box( buf, 0);
     file_cannot_open();
     return;
 }

 if( ( stbuf.st_mode & S_IFMT) != S_IFREG){
     sprintf( buf, "\"%s\" is not a regular file", filename);
     nc_show_alert_box( buf, 0);
     file_cannot_open();
     return;
 }

}

void file_cannot_open( void)
{
 ll_file_delete();

 if( ll_file_current == NULL) {
     nc_show_alert_box( "No more files", 0);
     do_exit( 1);
 }

 file_open( ll_file_current->filename);
}

void file_next_line( void)
{
 if( stbuf.st_size <= bytes_to_read) {
     term_beep();
     return;
 }

 if( (stbuf.st_size - fpos) < bytes_to_read || (fpos + bytes_to_read) == stbuf.st_size) {
      term_beep();
      return;
 }

 fpos = lseek( fd, ( long) bytes_per_row, SEEK_CUR);

 hex_showfile();
}

void file_prev_line( void)
{
 if( fpos < bytes_per_row) {
     term_beep();
     fpos = lseek( fd, (long) 0, SEEK_SET);
     hex_showfile();
     return;
 }

 fpos = lseek( fd, ( long) -1 * bytes_per_row, SEEK_CUR);
 hex_showfile();
}

void file_next_byte( void)
{
 if( stbuf.st_size <= bytes_to_read) {
     term_beep();
     return;
 }

 if( (stbuf.st_size - fpos) < bytes_to_read || (fpos + bytes_to_read) == stbuf.st_size) {
     term_beep();
     return;
 }
 
 fpos = lseek( fd, ( long) 1, SEEK_CUR);
 hex_showfile();
}

void file_prev_byte( void)
{

 if( fpos == 0) {
     term_beep();
     return;
 }

 fpos = lseek( fd, ( long) -1, SEEK_CUR);
 hex_showfile();
}

void file_go_home( void)
{
 if( fpos == 0) {
     term_beep();
     return;
 }

 fpos = lseek( fd, (long) 0, SEEK_SET);
 hex_showfile();
}

void file_go_end( void)
{
 if( stbuf.st_size <= bytes_to_read) {
     term_beep();
     return;
 }

 if( (stbuf.st_size - fpos) < bytes_to_read || (fpos + bytes_to_read) == stbuf.st_size) {
      term_beep();
      return;
 }

 fpos = lseek( fd, (long) (stbuf.st_size % bytes_to_read) * -1, SEEK_END);
 hex_showfile();
}

void file_next_block( void)
{
 if( stbuf.st_size <= bytes_to_read) {
     term_beep();
     return;
 }

 if( (stbuf.st_size - fpos) < bytes_to_read || (fpos + bytes_to_read) == stbuf.st_size) {
      term_beep();
      return;
 }

 fpos = lseek( fd, (long) bytes_to_read, SEEK_CUR);
 hex_showfile();
}

void file_prev_block( void)
{
 if( fpos < bytes_to_read) {
     term_beep();
     fpos = lseek( fd, (long) 0, SEEK_SET);
     hex_showfile();
     return;
 }

 fpos = lseek( fd, (long) -1 * bytes_to_read, SEEK_CUR);
 hex_showfile();
}

