/*
   Search functions
   --------------------------------------------------------------------
   VCHE RAW - Virtual Console Hex Editor Run Anywhere version

   Copyright (C) 1998 Diego Javier Grigna <diego@grigna.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"

char srch[ 64]; /* The string to search */
static long search_again_offset;
static long re_search;

static void search_string( char *str, long offset);
static long search_mem( char *sbuffer, char *pattern, long sizes, long sizep);
static void search_set_char_colorxy( char byte, int attr);

void search( void)
{
 raw_save_screen();

 raw_dialog_box( "Enter search string: ", srch, 38, VC_COLOR_BACKCYAN, VC_WHITE);

 term_goto( (scrn.cols - 38) / 2, scrn.lines / 2);
 key_scanf( srch, 38);

 strcpy( srch, trim_spaces( srch));

 ll_edit_head = NULL;

 if( srch[ 0] != 0 )
     search_string( srch, 0);

 raw_load_screen();
}

void search_again( void)
{
 search_again_offset = search_again_offset < fpos? fpos : search_again_offset;

 ll_edit_head = NULL;

 if( srch[ 0] != 0 )
     search_string( srch, search_again_offset);
}

void filegoto( void)
{
 char posstr[ 64];
 long newpos = -1;

 raw_save_screen();

 bzero( &posstr, sizeof( posstr));

 sprintf( posstr, "%ld", fpos);

 raw_dialog_box( "Enter file position: ", posstr, 38, VC_COLOR_BACKCYAN, VC_WHITE);

 term_goto( (scrn.cols - 38) / 2, scrn.lines / 2);
 key_scanf( posstr, 38);

 if( !strncasecmp( posstr, "0x", 2)) {
     sscanf( posstr, "%lx", &newpos);
 }
 else
     newpos = atol( posstr);

 if( newpos < 0 || newpos > stbuf.st_size) {
     raw_show_alert_box( "Invalid offset!", 5);
     raw_load_screen();
     return;
 }

 fpos = lseek( fd, (long) newpos, SEEK_SET);

 raw_load_screen();
}

static void search_string( char *str, long offset)
{
 struct ll_edit *node;
 struct ll_edit *tmp;
 char search_buf[ 8192];
 long filepos = 0;
 long bytes_read;
 long sizep = strlen( str);
 long retv;
 int i = 0;

 search_again_offset = 0;
 lseek( fd, (long) offset, SEEK_SET);

 raw_save_screen();
 raw_show_message( " Searching... ", 5);

 while( filepos < stbuf.st_size) {
        filepos = file_getpos();
        bytes_read = read( fd, &search_buf, sizeof( search_buf) - 1);
        if( bytes_read <= 0) 
            break;

        retv = search_mem( search_buf, str, bytes_read, sizep);

        if( retv >= 0) {
            fpos = filepos + retv;
            search_again_offset = fpos + sizep;
            while( i < sizep) {
                   node = ll_edit_head;
                   tmp  = ll_edit_head;

                   while( node != NULL) {
                          if( node->filepos < ( fpos + i)) {
                              tmp  = node;
                              node = node->next;
                          } else
                              break;
                   }

                   ll_edit_add( search_buf[ retv + i], fpos + i, tmp);
                   i++;
            }
            raw_load_screen();
            lseek( fd, (long) fpos, SEEK_SET);
            return;
        }
        if( retv == -2)
            lseek( fd, (long) filepos + re_search, SEEK_SET);
 }

 raw_show_alert_box( "String not found", 5);

 raw_load_screen();
 lseek( fd, (long) fpos, SEEK_SET);
}

static long search_mem( char *sbuffer, char *pattern, long sizes, long sizep)
{
 long found = 0;
 int i, j;

 i = j = 0;

 while( j < sizes && i < sizep && found < sizep) {

        if( sbuffer[ j] == pattern[ i]) {
            found++; 
            j++;
            i++;
        } else {
            j++;
            i = 0;
            found = 0;
        }
 }

 if( found == sizep)
     return j - found;

 if( found > 0) {
     re_search = j - found;
     return -2;
 }

 return -1;
}

void search_set_color( void)
{
 struct ll_edit *node = ll_edit_head;
 int bytes;

 while( node != NULL) {
        if( node->filepos < fpos)
            node = node->next;
        else
            break;
 }

 if( node != NULL) {
     bytes = node->filepos - fpos;
     while( bytes < bread && node != NULL && node->filepos <= (fpos + bytes)) {

          ey = bytes / bytes_per_row;
          ex = - ey * bytes_per_row + bytes;

          if( flag_nohex)
              search_set_char_colorxy( node->byte, VC_BG_LIGHTGRAY);
          else
              set_char_colorxy( node->byte, VC_BG_LIGHTGRAY);

          node = node->next;
          ex++;
          if( ex > bytes_per_row - 1) {
              ex = 0;
              ey++;
          }
          if( node != NULL)
              bytes = node->filepos - fpos;
     }

 }

 ll_edit_free();
}

static void search_set_char_colorxy( char byte, int attr)
{
 int posa;
 int ay, ax;
 int posxya;

 ay = ey + 1;
 ax = ex + scrn.cols - bytes_per_row;

 posxya = 4 + 2 *( ay * scrn.cols + ax);
 posa = posxya - (4 + 2 * scrn.cols);

 obuffer[ posa    ] = byte;
 obuffer[ posa + 1] = attr;
}

